
var Plate = function()
{
	var plate = {};
	plate.rows = 8;
	plate.columns = 12;
	plate.wells = [];
	plate.wellPainter = null;
	
	/**
		Initialize a plate with the given number of rows and columns.
	*/
	plate.init = function(rows, columns, wellPainter)
	{
		if (rows) plate.rows = rows;
		if (columns) plate.columns = columns;
		if (wellPainter) plate.wellPainter = wellPainter;
		
		for (var c = 0; c < plate.columns; c++)
		{
			for (var r = 0; r < plate.rows; r++)
			{
				plate.wells[plate.wells.length] = new Well(r, c);
			}
		}
	}
	
	/**
		Change the well painter for the plate.
	*/
	plate.setWellPainter = function(wellPainter)
	{
		plate.wellPainter = wellPainter;
	}
	
	/**
		Get the number of rows on the plate.
	*/
	plate.getRows = function()
	{
		return plate.rows;
	}

	/**
		Get the number of columns on the plate.
	*/
	plate.getColumns = function()
	{
		return plate.columns;
	}

	plate.getFirstEmptyColumn = function()
	{
		for (var column = 0; column < plate.columns; column++)
		{
			var colIsEmpty = true;
			var wellNo = column * plate.rows;
			for (var row = 0; row < plate.rows; row++)
			{
				if (plate.wells[wellNo+row].extract) break;
			}
			if (row == plate.rows) return column;
		}
		return -1;
	}
	
	/**
		Get all wells on the plate.
	*/
	plate.getWells = function()
	{
		return plate.wells;
	}
	
	/**
		Get all wells in the given row. First row is 0.
	*/
	plate.getRow = function(row)
	{
		var result = [];
		for (var i = row; result.length < plate.columns; i += plate.rows)
		{
			result[result.length] = plate.wells[i];
		}
		return result;
	}
	
	/**
		Get all wells in the given column. First column is 0.
	*/
	plate.getColumn = function(column)
	{
		var result = [];
		for (var i = column*plate.rows; result.length < plate.rows; i++)
		{
			result[result.length] = plate.wells[i];
		}
		return result;
	}

	/**
		Get all wells between the given start and end column (inclusive).
	*/
	plate.getPool = function(startCol, endCol)
	{
		var result = [];
		var numCols = endCol - startCol + 1;
		for (var i = startCol*plate.rows; result.length < plate.rows*numCols; i++)
		{
			result[result.length] = plate.wells[i];
		}
		return result;
	}

	
	/**
		Get the well at the given coordinate.
	*/
	plate.getWell = function(row, column)
	{
		return plate.wells[row + column * plate.rows];
	}
	
	plate.getWellByLocation = function(location)
	{
		var row = Reggie.alphaToWell(location.substring(0,1));
		var column = parseInt(location.substring(1))-1;
		return plate.getWell(row, column);
	}
	
	/**
		Get all wells that contains extract with a given name.
	*/
	plate.getWellsByName = function(name)
	{
		var result = [];
		for (var i = 0; i < plate.wells.length; i++)
		{
			var well = plate.wells[i];
			if (well.extract && well.extract.name == name)
			{
				result[result.length] = plate.wells[i];
			}
		}
		return result;
	}
	
	/**
		Get all wells that are selected.
	*/
	plate.getSelected = function()
	{
		var selected = [];
		for (var i = 0; i < plate.wells.length; i++)
		{
			var well = plate.wells[i];
			if (well.selected)
			{
				selected[selected.length] = well;
			}
		}
		return selected;
	}

	/**
		Set the selected status for all wells in the array.
		The wells are automatically repainted.
	*/
	plate.setSelected = function(wells, select)
	{
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			well.selected = select;
			well.paint(plate.wellPainter, plate.poolSchema);
		}
	}
	
	/**
		Get all wells which are used (well.extract has been set)
	*/
	plate.getUsed = function()
	{
		var used = [];
		for (var i = 0; i < plate.wells.length; i++)
		{
			var well = plate.wells[i];
			if (well.extract)
			{
				used[used.length] = well;
			}
		}
		return used;
	}
	
	/**
		Remove extract from all wells.
	*/
	plate.clear = function(wells, clearSelected)
	{
		if (wells.length == 0) return;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			well.extract = null;
			well.poolNum = -1;
			if (clearSelected) well.selected = false;
		}
	}
	
	/**
		Toggle the selected status for all wells in the array.
		The next status is determined by the current status of 
		the first well in the array.
	*/
	plate.toggleSelected = function(wells)
	{
		if (wells.length == 0) return;
		plate.setSelected(wells, !wells[0].selected);
	}
	
	/**
		Test the filter function on all wells and select or de-select
		the well if it matches/not matches.
	*/
	plate.selectByFilter = function(wells, filter)
	{
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			well.selected = filter(well);
		}
		plate.paint(wells);
	}
	
	/**
		Enable or disable highlight of the given wells.
	*/
	plate.setHighlight = function(wells, highlightClass, on)
	{
		var tmpClass = on ? highlightClass : null;
		for (var i = 0; i < wells.length; i++)
		{
			var well = wells[i];
			well.highlightClass = tmpClass;
			Doc.addOrRemoveClass(well.tag, highlightClass, on);
		}
	}
	
	/**
		Paint the given wells.
	*/
	plate.paint = function(wells)
	{
		for (var i = 0; i < wells.length; i++)
		{
			wells[i].paint(plate.wellPainter);
		}
	}
	
	return plate;
}();

/**
	Represents a well on the plate. Each well is created when 
	the page is loaded and is initially empty.
*/
function Well(row, column)
{
	this.row = row;
	this.column = column;
	this.poolNum = -1;
	this.selected = false;
	this.highlighClass = null;
	this.extract = null;
	this.warning = null;
	this.error = null;
	
	this.tag = document.getElementById('well.'+row+'.'+column);
}

/**
	Set the extract item that is put into this well. Use
	'null' to clear the well. The extract should always
	have a 'name' property, and an 'id' property if it 
	exists in the database.
*/
Well.prototype.setExtract = function(extract)
{
	this.extract = extract;
}

Well.prototype.setWarning = function(warning)
{
	this.warning = warning;
}

Well.prototype.hasWarning = function()
{
	return this.warning != null || this.extract && this.extract.warning != null;
}

Well.prototype.setError = function(error)
{
	this.error = error;
}

Well.prototype.hasError = function()
{
	return this.error != null || this.extract && this.extract.error != null;
}

Well.prototype.getClassName = function(wellPainter)
{
	var c = this.column;
	var cls = 'well col-'+c;
	if (this.selected) cls += ' selected';
	if (!this.extract) cls += ' empty';
	if (this.hasError()) cls += ' err';
	if (this.hasWarning()) cls += ' warning';
	if (this.highlightClass) cls += ' ' + this.highlightClass;

	if (wellPainter)
	{
		cls += ' ' + wellPainter.getClassNameForWell(this);
	}
	return cls;
}

Well.prototype.getText = function(wellPainter)
{
	var text = '';
	if (wellPainter)
	{
		text = wellPainter.getWellText(this);
	}
	else
	{
		if (this.extract)
		{
			text += '<div class="name">'+Strings.encodeTags(this.extract.name)+'</div>'
		}
	}
	if (this.hasError())
	{
		text += '<div class="err-msg">'+(this.error || this.extract.error)+'</div>';
	}
	if (this.hasWarning())
	{
		text += '<div class="warn-msg">'+(this.warning || this.extract.warning)+'</div>';
	}
	return text;
}

Well.prototype.paint = function(wellPainter)
{
	this.warning = null;
	this.error = null;
	this.tag.innerHTML = this.getText(wellPainter);
	this.tag.className = this.getClassName(wellPainter);
}

/**
	Helper class for counting items with the same 'key' value.
	The 'key' that has the highest count is stored in 'maxKey'
	and the count for that key in 'maxCount'. All other keys
	and counts are found in the 'counts' map.
 */
function ItemCounter()
{
	this.numCounted = 0;
	this.maxCount = 0;
	this.maxKey = '';
	this.counts = {};
}

ItemCounter.prototype.count = function(key)
{
	if (!key) return;
	this.numCounted++;
	var c = this.counts[key];
	c = c ? c+1 : 1;
	this.counts[key] = c;
	if (c > this.maxCount)
	{
		this.maxKey = key;
		this.maxCount = c;
	}
}

ItemCounter.prototype.numKeys = function()
{
	return Object.keys(this.counts).length;
}

var MipsColor = function()
{
	var color = {};
	
	var PRESET = { 
		'ID': 'bg-green', 
		'B1B2': 'bg-blue'
	};
	var MORE_COLORS = ['bg-yellow', 'bg-gray', 'bg-green-dk', 'bg-blue-dk', 'bg-yellow-dk'];
	
	var nextColor = 0;
	var colorMap = {};
	
	color.reset = function()
	{
		colorMap = {};
		nextColor = 0;
	}
	
	/*
		Get a color preset for the given MIPs panel. A preset is selected
		if the mipsPanel starts with the name of one of the entries in PRESET.
		For example: ID_v1 --> ID --> bg-green
	*/
	color.getPresetColor = function(mipsPanel)
	{
		for (var p in PRESET)
		{
			if (mipsPanel.indexOf(p) == 0)
			{
				return PRESET[p];
			}
		}
		return null;
	}
	
	/**
		Get a color for the given MIPs panel. If a color has already been
		assigned, the same color is returned. Otherwise, we check if the MIPs
		panel is one of the presets. If not, a color is assigned from
		MORE_COLORS in order.
	*/
	color.getColor = function(mipsPanel)
	{
		var c = colorMap[mipsPanel];
		if (!c) 
		{
			c = color.getPresetColor(mipsPanel);
			if (!c)
			{
				c = MORE_COLORS[nextColor++];
				if (nextColor >= MORE_COLORS.length) nextColor = 0;
			}
			colorMap[mipsPanel] = c;
		}
		return c;
	}
	
	return color;
}();
